/*
** Copyright (c) 2003 CA.  All rights reserved.  
** This software may not be duplicated, disclosed or reproduced in whole or 
** in part for any purpose except as authorized by the applicable license agreement, 
** without the express written authorization of CA. All authorized reproductions 
** must be marked with this language.
**
** TO THE EXTENT PERMITTED BY APPLICABLE LAW, CA PROVIDES THIS 
** SOFTWARE WITHOUT WARRANTY OF ANY KIND, INCLUDING 
** WITHOUT LIMITATION, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, 
** FITNESS FOR A PARTICULAR PURPOSE OR NONINFRINGEMENT.  IN NO EVENT 
** WILL CA BE LIABLE TO THE END USER OR ANY THIRD PARTY FOR ANY LOSS 
** OR DAMAGE, DIRECT OR INDIRECT, FROM THE USE OF THIS MATERIAL, 
** INCLUDING WITHOUT LIMITATION, LOST PROFITS, BUSINESS 
** INTERRUPTION, GOODWILL, OR LOST DATA, EVEN IF CA IS EXPRESSLY 
** ADVISED OF SUCH LOSS OR DAMAGE.
*/

// SiteMinder Java Authentication API sample.

package com.netegrity.sdk.javaauthapi;

import java.util.Date;
import java.text.DateFormat;
import com.netegrity.policyserver.smapi.*;


/**
 * A simple SiteMinder Java Authentication Scheme.
 */

public class AuthApiSample implements SmAuthScheme  {
    private static final int SCHEME_VERSION = SmAuthQueryResponse.SMAUTH_API_VERSION_V3;
    private static final String SCHEME_DESCRIPTION = "SiteMinder(tm) Example Java authentication scheme, (c) Netegrity 2003";


    /**
     * Returns information about the authentication scheme.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @param request - The request code, SMAUTH_QUERY_DESCRIPTION or SMAUTH_QUERY_CREDENTIALS_REQ.
     * @param response - Holds methods by which query() returns the requested information.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */

    public SmAuthStatus query(String parameter, String secret, SmAuthQueryCode request, SmAuthQueryResponse response) {
        if (null == response) {
            return SmAuthStatus.SMAUTH_FAILURE;
        }

        if (SmAuthQueryCode.SMAUTH_QUERY_DESCRIPTION == request) {
            response.setResponseBuffer(SCHEME_DESCRIPTION);
            response.setResponseCode(SCHEME_VERSION);
        }
        else if (SmAuthQueryCode.SMAUTH_QUERY_CREDENTIALS_REQ == request) {
            //response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_BASIC);
            response.setResponseCode(SmAuthQueryResponse.SMAUTH_CRED_X509CERT | SmAuthQueryResponse.SMAUTH_CRED_X509CERT_USER_DN);

        }
        else {
            return SmAuthStatus.SMAUTH_FAILURE;
        }

        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    /**
     * SiteMinder invokes this method so the authentication scheme can
     * perform its own initialization procedure. This method is invoked once
     * for each authentication scheme instance when it is first loaded.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */

    public SmAuthStatus  init(String parameter, String secret)  {
        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    /**
     * SiteMinder invokes this method during shutdown so the authentication
     * scheme can perform its own rundown procedure. This method is invoked
     * once for each authentication scheme instance during SiteMinder shutdown.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @return: If successful returns SMAUTH_SUCCESS, otherwise returns SMAUTH_FAILURE
     */

    public SmAuthStatus release(String parameter, String secret) {
        return SmAuthStatus.SMAUTH_SUCCESS;
    }


    /**
     * SiteMinder invokes this method to authenticate user credentials.
     *
     * @param parameter - The parameter string as specified for the authentication scheme.
     * @param secret - The secret string as specified for the authentication scheme.
     * @param challengeReason - The reason for the original authentication challenge, or 0 if unknown.
     * @param context - Contains request context and methods to return message buffers.
     *
     * @return: an SmAuthenticationResult object.
     */

    void log(SmAuthenticationContext context, String msg) {
         context.getAPIContext().trace(this.getClass().getName(), "THE API Version is::::: ['" + context.getAPIContext().getVersion() +"']");
    }

    public SmAuthenticationResult authenticate(String parameter, String secret, int challengeReason,  SmAuthenticationContext context) {
        // cannot do authentication without the authentication context
        if (null == context) {
            return new SmAuthenticationResult(SmAuthStatus.SMAUTH_NO_USER_CONTEXT, SmAuthenticationResult.REASON_NONE);
        }

    	context.getAPIContext().trace(this.getClass().getName(), ">>>>>>>>>>authenticate");
    	context.getAPIContext().trace(this.getClass().getName(), "THE API Version is::::: ['" + context.getAPIContext().getVersion() +"']");

        UserCredentialsContext userCredentialsContext = context.getUserCredentialsContext();

        if ( userCredentialsContext == null) {
                log(context, "user Credentials are null");
        }
        else {
              byte[] bytes = userCredentialsContext.getCertAsBinary();
              log(context, "user Credentials cert as " + bytes.length);

              String dn = userCredentialsContext.getCertUserDN();
              log(context, "user Credentials DN " + dn);
        }
        // If the scheme is designed not to disambiguate users, it should return SmAuthApi_NoUserContext.
        UserContext theUserContext = context.getUserContext();

        if ((null == theUserContext) || !theUserContext.isUserContext()) {
            return  new SmAuthenticationResult(SmAuthStatus.SMAUTH_NO_USER_CONTEXT, SmAuthenticationResult.REASON_NONE);
        }

        // Reject the user if the password is not entered.
        UserCredentialsContext theUserCredentialsContext = context.getUserCredentialsContext();

        if (null == theUserCredentialsContext)  {
            return
                new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }

        String thePassword = theUserCredentialsContext.getPassword();

        if (thePassword.length() <= 0) {
            return
                new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }

        // Check if the user account is disabled.
        try {
            if (0 != Integer.parseInt(theUserContext.getProp("disabled")))  {
                context.setUserText("User account is disabled.");

                return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_USER_DISABLED);
            }
        }
        catch (NumberFormatException exc) {
            // Do nothing -- the user is not disabled
        }

        // authenticate the user
        String authUserText;

        try {
            authUserText = theUserContext.authenticateUser(thePassword);
        }
        catch (Throwable exc)  {
            // insure subsequent code knows the authentication attempt failed
            authUserText = null;
        }

        if (null == authUserText) {
            context.setErrorText("Unable to authenticate user " + theUserContext.getUserName());

            return new SmAuthenticationResult(SmAuthStatus.SMAUTH_REJECT, SmAuthenticationResult.REASON_NONE);
        }

        // Set the time stamp of the user's last authentication.
        // For demonstration purposes, we will store the last login
        // in the "PIN" user property as a printable date&time string.

        String timeString = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,DateFormat.MEDIUM).format(new Date());

        // put single quotes around the string if the user directory is "ODBC:"
        String theNameSpace = theUserContext.getDirNameSpace();

        if ((theNameSpace != null) && theNameSpace.equals("ODBC:")) {
            timeString = "'" + timeString + "'" ;
        }

        if (0 != theUserContext.setProp("pin", timeString)) {
            context.setUserText("Failed to set the time stamp for user's profile attribute " + parameter);
        }

        // If a parameter is supplied, set it as app specific data
        if (parameter != null) {
            try {
                APIContext apiContext = context.getAPIContext();
                AppSpecificContext appContext = apiContext.getAppSpecificContext();
                appContext.setData(parameter.getBytes());
            }
            catch (NullPointerException exc) {
                context.setUserText("Failed to modify application specific context");
            }
            catch (SmJavaApiException exc) {
                context.setUserText("Failed to modify application specific context");
            }
        }

        return new SmAuthenticationResult(SmAuthStatus.SMAUTH_ACCEPT, SmAuthenticationResult.REASON_NONE);
    }
}

// EOF AuthApiSample.java
